from Acquisition import aq_inner
from Acquisition import aq_parent
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import plone
from ftw.trash.interfaces import IRestorable
from ftw.trash.interfaces import ITrashed
from ftw.trash.tests import duplicate_with_dexterity
from ftw.trash.tests import FunctionalTestCase
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import isLinked
from zExceptions import Unauthorized


@duplicate_with_dexterity
class TestDeletion(FunctionalTestCase):

    @browsing
    def test_deleted_content_is_marked_with_IRestorable(self, browser):
        catalog = getToolByName(self.layer['portal'], 'portal_catalog')
        self.grant('Contributor')

        folder = create(Builder('folder').within(create(Builder('folder'))))
        subfolder = create(Builder('folder').within(folder))
        self.assertIn(folder.getId(), aq_parent(aq_inner(folder)).objectIds())
        self.assertEqual(3, len(catalog.unrestrictedSearchResults()))
        self.assert_provides(folder, None)
        self.assert_provides(subfolder, None)

        browser.login().visit(folder)
        browser.click_on('Delete')
        self.assertEquals(
            'Do you really want to delete this folder and all its contents?'
            ' (This will delete a total of 2 items.)',
            plone.first_heading())
        self.assert_provides(folder, None)
        self.assert_provides(subfolder, None)

        browser.click_on('Delete')
        self.assertIn(folder.getId(), aq_parent(aq_inner(folder)).objectIds())
        self.assertEqual(3, len(catalog.unrestrictedSearchResults()))
        self.assert_provides(folder, IRestorable, ITrashed)
        self.assert_provides(subfolder, ITrashed)

    @browsing
    def test_children_of_site_root_are_trashed_instead_of_deleted(self, browser):
        catalog = getToolByName(self.layer['portal'], 'portal_catalog')
        self.grant('Contributor')

        folder = create(Builder('folder'))
        self.assertIn(folder.getId(), aq_parent(aq_inner(folder)).objectIds())
        self.assertEqual(1, len(catalog.unrestrictedSearchResults()))
        self.assert_provides(folder, None)

        browser.login().visit(folder)
        browser.click_on('Delete')

        self.assertEquals('Do you really want to delete this folder and all its contents?',
                          plone.first_heading())
        browser.click_on('Delete')
        self.assertIn(folder.getId(), aq_parent(aq_inner(folder)).objectIds())
        self.assertEqual(1, len(catalog.unrestrictedSearchResults()))
        self.assert_provides(folder, IRestorable, ITrashed)

    def test_manage_delObjects_requires_both_delete_permissions(self):
        """collective.deletepermission adapts the behavior so that a user needs both,
        "Delete objects" on the parent and "Delete portal content" on the child.

        We dont want to change security and thus want the permissions to protect our
        trashing.

        This test must work accordingly to
        ftw.trash.tests.test_installation.TestTrashNotInstalled.[test name]
        """

        self.grant('Manager')
        parent = create(Builder('folder'))
        parent.manage_permission('Delete objects', roles=['Contributor'], acquire=False)
        child = create(Builder('folder').within(parent))
        child.manage_permission('Delete portal content', roles=['Contributor'], acquire=False)
        self.assertIn(child.getId(), parent.objectIds())
        self.assert_provides(parent, None)
        self.assert_provides(child, None)

        with self.assertRaises(Unauthorized):
            # Our test user with role Manager is not allowed to delete the child because
            # we have limited the permissions to "Contributor".
            parent.manage_delObjects([child.getId()])
            self.assertIn(child.getId(), parent.objectIds())
            self.assert_provides(parent, None)
            self.assert_provides(child, None)

        user = create(Builder('user').with_roles('Contributor', on=parent))
        with self.user(user):
            parent.manage_delObjects([child.getId()])
            self.assertIn(child.getId(), parent.objectIds())
            self.assert_provides(parent, None)
            self.assert_provides(child, IRestorable, ITrashed)

    def test_confirmation_dialog_link_integrity_checker_should_actually_delete_the_object(self):
        """The confirmation dialog deletes the object within a later rolled-back savepoint
        in order to detect broken links.
        If we only trash the object, the link checker will no longer work.
        Therefore ftw.trash must back off while within the link integrity checker.
        """
        if self.is_dexterity:
            # the test does not work with dexterity.
            return

        self.grant('Site Administrator')
        target = create(Builder('folder'))
        # be aware that the page object is always archetypes.
        create(Builder('page')
               .having(text='<p><a href="resolveuid/{}">folder</a>'.format(IUUID(target))))

        self.assertTrue(isLinked(target))

    def test_trash_provides_manage_immediatelyDeleteObjects_method_on_portal(self):
        self.grant('Site Administrator')
        folder_id = create(Builder('folder')).getId()
        self.assertIn(folder_id, self.portal.objectIds())
        self.portal.manage_immediatelyDeleteObjects(folder_id)
        self.assertNotIn(folder_id, self.portal.objectIds())

    def test_trash_provides_manage_immediatelyDeleteObjects_method_on_folder(self):
        self.grant('Site Administrator')
        parent = create(Builder('folder').titled(u'Parent'))
        folder_id = create(Builder('folder').within(parent)).getId()
        self.assertIn(folder_id, parent.objectIds())
        parent.manage_immediatelyDeleteObjects(folder_id)
        self.assertNotIn(folder_id, parent.objectIds())



from ftw.trash.interfaces import ITrashed
from plone.indexer import indexer
from zope.interface import Interface


@indexer(Interface)
def trashed_indexer(obj):
    """Indexer for the `trashed` index, this index is used to filter trashed
    objects from catalog search results by default.
    """
    return ITrashed.providedBy(obj)



from ftw.trash.interfaces import ITrashed
from ftw.trash.trasher import Trasher
from ftw.trash.utils import called_from_ZMI
from ftw.trash.utils import is_trash_profile_installed
from ftw.trash.utils import within_link_integrity_check


def manage_trashObjects(self, ids=None, REQUEST=None):
    """Marks objects as trashed.
    """
    if ids is None:
        ids = []
    if isinstance(ids, basestring):
        ids = [ids]
    for id_ in ids:
        Trasher(self._getOb(id_)).trash()


def manage_delObjects(self, ids=None, REQUEST=None):
    if is_trash_profile_installed() and \
       not within_link_integrity_check() and \
       not called_from_ZMI(REQUEST):
        return self.manage_trashObjects(ids=ids, REQUEST=REQUEST)
    else:
        return self.manage_immediatelyDeleteObjects(ids=ids, REQUEST=REQUEST)


def manage_immediatelyDeleteObjects(self, ids=None, REQUEST=None):
    """Immediately delete an object instead of only trashing it.
    """
    return self._old_manage_delObjects(ids=ids, REQUEST=REQUEST)


def searchResults(self, REQUEST=None, **kw):
    kw = kw.copy()

    # Meaning of indexed values of 'trashed':
    # - True => object is trashed
    # - False => object is not trashed
    # - None => object is not properly indexed -> treat like False

    # Meaing and defaults when querying 'trashed':
    # - True => only return trashed objects
    # - False => return trashed objects (includes not properly indexed objects)
    # - None => return all objects, do not filter
    kw.setdefault('trashed', False)

    if kw['trashed'] is False:
        kw['trashed'] = [False, None]
    elif kw['trashed'] is None:
        kw['trashed'] = [True, False, None]

    return self._old_searchResults(REQUEST, **kw)


def _getFieldObjects(self, *args, **kwargs):
    return filter(lambda obj: not ITrashed.providedBy(obj),
                  self._old__getFieldObjects(*args, **kwargs))


def getRawActionAdapter(self, *args, **kwargs):
    adapter_ids = self._old_getRawActionAdapter(*args, **kwargs)
    adapter_result = []

    for adapter_id in adapter_ids:
        if not ITrashed.providedBy(self.get(adapter_id)):
            adapter_result.append(adapter_id)

    return tuple(adapter_result)



from AccessControl import getSecurityManager
from ftw.trash import _
from ftw.trash.interfaces import ITrashed
from Products.CMFPlone.utils import safe_unicode
from Products.statusmessages.interfaces import IStatusMessage
from zExceptions import NotFound


def prevent_accessing_trashed_content_after_traversal(event):
    context = event.request.PARENTS[0]
    if not ITrashed.providedBy(context):
        return

    if getSecurityManager().getUser().has_role('Manager'):
        IStatusMessage(event.request).addStatusMessage(
            _(u'statusmessage_content_trashed',
              default=u'The content "${title}" is trashed.',
              mapping={u'title': safe_unicode(context.Title())}),
            type='warning')
        return

    raise NotFound()



from AccessControl import getSecurityManager
from AccessControl.requestmethod import postonly
from Acquisition import aq_inner
from Acquisition import aq_parent
from ftw.trash import _
from ftw.trash.interfaces import IRestorable
from ftw.trash.interfaces import ITrashed
from ftw.trash.trasher import Trasher
from ftw.trash.utils import filter_children_in_paths
from itertools import imap
from plone import api
from plone.protect import CheckAuthenticator
from plone.protect import protect
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_unicode
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.statusmessages.interfaces import IStatusMessage
from zExceptions import BadRequest
from zExceptions import Unauthorized
from zope.component.hooks import getSite


class TrashView(BrowserView):
    tempalte = ViewPageTemplateFile('templates/trash.pt')
    clean_confirmation_tempalte = ViewPageTemplateFile('templates/clean-trash-confirmation.pt')

    max_amount_of_items = 1000

    def __call__(self):
        self.request.set('disable_border', '1')
        self.portal_path = '/'.join(getSite().getPhysicalPath())
        self.can_clean_trash = getSecurityManager().checkPermission('Clean trash', self.context)
        return self.tempalte()

    @postonly
    @protect(CheckAuthenticator)
    def restore(self, REQUEST, uuid):
        """Restore an item by uid.
        """
        catalog = getToolByName(self.context, 'portal_catalog')
        brains = catalog({'UID': uuid, 'trashed': True})
        if len(brains) != 1:
            raise BadRequest()

        obj = brains[0].getObject()
        parent = aq_parent(aq_inner(obj))
        if ITrashed.providedBy(parent):
            IStatusMessage(self.request).addStatusMessage(
                _(u'statusmessage_content_restore_error_parent_trashed',
                  default=(u'"${title}" cannot be restored because the parent container'
                           u' "${parent}" is also trashed.'
                           u' You need to restore the parent first.'),
                  mapping={u'title': safe_unicode(obj.Title()),
                           u'parent': safe_unicode(parent.Title())}),
                type='error')
            return self.request.response.redirect(self.context.absolute_url() + '/trash')

        trasher = Trasher(obj)
        if not trasher.is_restorable():
            IStatusMessage(self.request).addStatusMessage(
                _(u'statusmessage_content_restore_not_allowed',
                  default=(u'You are not allowed to restore "${title}".'
                           u' You may need to change the workflow state of the parent content.'),
                  mapping={u'title': safe_unicode(obj.Title())}),
                type='error')
            return self.request.response.redirect(self.context.absolute_url() + '/trash')

        trasher.restore()
        IStatusMessage(self.request).addStatusMessage(
            _(u'statusmessage_content_restored',
              default=u'The content "${title}" has been restored.',
              mapping={u'title': safe_unicode(obj.Title())}),
            type='info')

        target_url = obj.restrictedTraverse('@@plone_context_state').view_url()
        self.request.response.redirect(target_url)

    def confirm_clean_trash(self):
        """Show confirmation dialog for cleaning the trash.
        """
        self.request.set('disable_border', '1')
        return self.clean_confirmation_tempalte()

    @postonly
    @protect(CheckAuthenticator)
    def clean_trash(self, REQUEST):
        """Clean the trash by permantly deleting all trashed objects.
        """
        if self.request.form.get('cancel'):
            return self.request.response.redirect(self.context.absolute_url() + '/trash')

        elif self.request.form.get('delete'):
            if not getSecurityManager().checkPermission('Clean trash', self.context):
                raise Unauthorized()

            catalog = getToolByName(self.context, 'portal_catalog')
            query = {'object_provides': IRestorable.__identifier__, 'trashed': True}

            paths_to_delete = filter_children_in_paths(
                [brain.getPath() for brain in catalog(query)])

            # The user maybe able to clean the trash, but depending on the workflow the user may not have
            # delete permission in the given context
            with api.env.adopt_roles(['Manager']):
                for path in paths_to_delete:
                    obj = self.context.restrictedTraverse(path)
                    got_path = '/'.join(obj.getPhysicalPath())
                    if got_path != path:
                        raise ValueError('Unexpectly found path {!r} when looking for {!r}'.format(
                            got_path, path))

                    aq_parent(aq_inner(obj)).manage_immediatelyDeleteObjects([obj.getId()])

            return self.request.response.redirect(self.context.absolute_url() + '/trash')

        else:
            raise BadRequest()

    def get_trashed_items(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        query = {
            'object_provides': IRestorable.__identifier__,
            'trashed': True,
            'sort_on': 'modified',
            'sort_order': 'reverse',
            'sort_limit': self.max_amount_of_items}

        return imap(self._brain_to_item, catalog(query)[:self.max_amount_of_items])

    def _brain_to_item(self, brain):
        return {
            'type': brain.Type,
            'modified': self.context.toLocalizedTime(brain.modified, long_format=1),
            'title': brain.Title,
            'location': brain.getURL(),
            'uuid': brain.UID}



from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Acquisition import aq_parent
from collective.deletepermission.del_object import protect_del_objects
from ftw.trash.events import BeforeObjectRestoredEvent
from ftw.trash.events import BeforeObjectTrashedEvent
from ftw.trash.events import ObjectRestoredEvent
from ftw.trash.events import ObjectTrashedEvent
from ftw.trash.exceptions import NotRestorable
from ftw.trash.interfaces import IIsRestoreAllowedAdapter
from ftw.trash.interfaces import IRestorable
from ftw.trash.interfaces import ITrashed
from zExceptions import Unauthorized
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.event import notify
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import noLongerProvides


class Trasher(object):
    """The trasher manages trashing, restoring and deleting of objects.
    """

    def __init__(self, context):
        self.context = context

    def trash(self):
        notify(BeforeObjectTrashedEvent(self.context))
        self._trash_recursive(self.context, is_root=True)
        notify(ObjectTrashedEvent(self.context))

    def is_restorable(self):
        parent = aq_parent(aq_inner(self.context))
        if ITrashed.providedBy(parent):
            # Cannot restore an object when the parent is still trashed.
            return False

        if not getMultiAdapter((self.context, self.context.REQUEST), IIsRestoreAllowedAdapter):
            return False

        return ITrashed.providedBy(self.context)

    def restore(self):
        if not self.is_restorable():
            raise NotRestorable()

        if not getSecurityManager().checkPermission('Restore trashed content', self.context):
            raise Unauthorized()

        notify(BeforeObjectRestoredEvent(self.context))
        self._restore_recursive(self.context, is_root=True)
        notify(ObjectRestoredEvent(self.context))

    def _trash_recursive(self, obj, is_root=False):
        protect_del_objects(aq_parent(aq_inner(obj)), obj.getId())
        alsoProvides(obj, ITrashed)
        if is_root:
            alsoProvides(obj, IRestorable)

        obj.setModificationDate()
        obj.reindexObject(idxs=['object_provides', 'trashed', 'modified'])
        map(self._trash_recursive, obj.objectValues())

    def _restore_recursive(self, obj, is_root=False):
        if not is_root and IRestorable.providedBy(obj):
            # This child was deleted separately; stop restore recursion.
            return

        noLongerProvides(obj, ITrashed)
        noLongerProvides(obj, IRestorable)
        obj.setModificationDate()
        obj.reindexObject(idxs=['object_provides', 'trashed', 'modified'])
        map(self._restore_recursive, obj.objectValues())


@implementer(IIsRestoreAllowedAdapter)
@adapter(Interface, Interface)
def default_is_restore_allowed(context, request):
    """The default IIsRestoreAllowedAdapter requires add permission on the parent
    object for restoring objects, as the action can be compared to adding new content.
    """
    parent = aq_parent(aq_inner(context))
    return bool(getSecurityManager().checkPermission('Add portal content', parent))



from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import statusmessages
from ftw.trash.tests import duplicate_with_dexterity
from ftw.trash.tests import FunctionalTestCase
from ftw.trash.trasher import Trasher
import transaction


@duplicate_with_dexterity
class TestTraversing(FunctionalTestCase):

    @browsing
    def test_browsing_trashed_content_raises_404(self, browser):
        self.grant('Contributor')

        parent = create(Builder('folder'))
        folder = create(Builder('folder').within(parent))
        subfolder = create(Builder('folder').within(folder))

        browser.login()
        # the user can access all content, since none is trashed
        browser.open(parent)
        browser.open(folder)
        browser.open(subfolder)

        # when we trash "folder", the parent is still accessible, but not the trashed content
        Trasher(folder).trash()
        transaction.commit()

        browser.open(parent)

        with browser.expect_http_error(404):
            browser.open(folder)

        with browser.expect_http_error(404):
            browser.open(subfolder)

    @browsing
    def test_allow_Manager_to_browse_trashed_content_with_status_message(self, browser):
        self.grant('Manager')

        folder = create(Builder('folder').titled(u'Fancy Folder'))
        browser.login()
        browser.open(folder)

        Trasher(folder).trash()
        transaction.commit()
        browser.open(folder)

        statusmessages.assert_message('The content "Fancy Folder" is trashed.')



from ftw.trash.interfaces import IBeforeObjectRestoredEvent
from ftw.trash.interfaces import IBeforeObjectTrashedEvent
from ftw.trash.interfaces import IObjectRestoredEvent
from ftw.trash.interfaces import IObjectTrashedEvent
from zope.component.interfaces import ObjectEvent
from zope.interface import implementer


@implementer(IBeforeObjectTrashedEvent)
class BeforeObjectTrashedEvent(ObjectEvent):
    """An object will be trashed.
    """

    def __init__(self, object):
        self.object = object


@implementer(IObjectTrashedEvent)
class ObjectTrashedEvent(ObjectEvent):
    """An object has been trashed.
    """

    def __init__(self, object):
        self.object = object


@implementer(IBeforeObjectRestoredEvent)
class BeforeObjectRestoredEvent(ObjectEvent):
    """An object will be restored.
    """

    def __init__(self, object):
        self.object = object


@implementer(IObjectRestoredEvent)
class ObjectRestoredEvent(ObjectEvent):
    """An object has been restored.
    """

    def __init__(self, object):
        self.object = object



from datetime import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.testing import freeze
from ftw.trash.exceptions import NotRestorable
from ftw.trash.interfaces import IBeforeObjectRestoredEvent
from ftw.trash.interfaces import IBeforeObjectTrashedEvent
from ftw.trash.interfaces import IIsRestoreAllowedAdapter
from ftw.trash.interfaces import IObjectRestoredEvent
from ftw.trash.interfaces import IObjectTrashedEvent
from ftw.trash.interfaces import IRestorable
from ftw.trash.interfaces import ITrashed
from ftw.trash.tests import duplicate_with_dexterity
from ftw.trash.tests import FunctionalTestCase
from ftw.trash.trasher import Trasher
from zExceptions import Unauthorized
from zope.component import adapter
from zope.component import getMultiAdapter
from zope.component import getSiteManager
from zope.interface import implementer
from zope.interface import Interface


@duplicate_with_dexterity
class TestTrasher(FunctionalTestCase):

    def test_marks_trashed_page_as_trashed_and_restorable(self):
        self.grant('Contributor')

        folder = create(Builder('folder'))
        self.assert_provides(folder, None)

        Trasher(folder).trash()
        self.assert_provides(folder, IRestorable, ITrashed)

    def test_marks_children_of_trashed_folder_only_as_trashed(self):
        self.grant('Contributor')

        folder = create(Builder('folder'))
        subfolder = create(Builder('folder').within(folder))
        self.assert_provides(folder, None)
        self.assert_provides(subfolder, None)

        Trasher(folder).trash()
        self.assert_provides(folder, IRestorable, ITrashed)
        self.assert_provides(subfolder, ITrashed)

    def test_catalog_is_updated_when_trashing_content(self):
        self.grant('Contributor')

        folder = create(Builder('folder'))
        subfolder = create(Builder('folder').within(folder))

        self.assertFalse(self.get_catalog_indexdata(folder).get('trashed'))
        self.assertNotIn(ITrashed.__identifier__,
                         self.get_catalog_indexdata(folder).get('object_provides'))
        self.assertNotIn(IRestorable.__identifier__,
                         self.get_catalog_indexdata(folder).get('object_provides'))

        self.assertFalse(self.get_catalog_indexdata(subfolder).get('trashed'))
        self.assertNotIn(ITrashed.__identifier__,
                         self.get_catalog_indexdata(subfolder).get('object_provides'))
        self.assertNotIn(IRestorable.__identifier__,
                         self.get_catalog_indexdata(subfolder).get('object_provides'))

        Trasher(folder).trash()
        self.assertTrue(self.get_catalog_indexdata(folder).get('trashed'))
        self.assertIn(ITrashed.__identifier__,
                      self.get_catalog_indexdata(folder).get('object_provides'))
        self.assertIn(IRestorable.__identifier__,
                      self.get_catalog_indexdata(folder).get('object_provides'))

        self.assertTrue(self.get_catalog_indexdata(subfolder).get('trashed'))
        self.assertIn(ITrashed.__identifier__,
                      self.get_catalog_indexdata(subfolder).get('object_provides'))
        self.assertNotIn(IRestorable.__identifier__,
                         self.get_catalog_indexdata(subfolder).get('object_provides'))

    def test_restorable_content_can_be_restored(self):
        self.grant('Site Administrator')

        folder = create(Builder('folder'))
        self.assert_provides(folder, None)

        Trasher(folder).trash()
        self.assert_provides(folder, IRestorable, ITrashed)

        Trasher(folder).restore()
        self.assert_provides(folder, None)

    def test_children_are_restored_correctly(self):
        self.grant('Site Administrator')

        folder = create(Builder('folder'))
        subfolder = create(Builder('folder').within(folder))
        self.assert_provides(folder, None)
        self.assert_provides(subfolder, None)

        Trasher(folder).trash()
        self.assert_provides(folder, IRestorable, ITrashed)
        self.assert_provides(subfolder, ITrashed)

        Trasher(folder).restore()
        self.assert_provides(folder, None)
        self.assert_provides(subfolder, None)

    def test_catalog_is_updated_when_restoring_content(self):
        self.grant('Site Administrator')

        folder = create(Builder('folder'))
        subfolder = create(Builder('folder').within(folder))

        self.assertFalse(self.get_catalog_indexdata(folder).get('trashed'))
        self.assertNotIn(ITrashed.__identifier__,
                         self.get_catalog_indexdata(folder).get('object_provides'))
        self.assertNotIn(IRestorable.__identifier__,
                         self.get_catalog_indexdata(folder).get('object_provides'))

        self.assertFalse(self.get_catalog_indexdata(subfolder).get('trashed'))
        self.assertNotIn(ITrashed.__identifier__,
                         self.get_catalog_indexdata(subfolder).get('object_provides'))
        self.assertNotIn(IRestorable.__identifier__,
                         self.get_catalog_indexdata(subfolder).get('object_provides'))

        Trasher(folder).trash()
        self.assertTrue(self.get_catalog_indexdata(folder).get('trashed'))
        self.assertIn(ITrashed.__identifier__,
                      self.get_catalog_indexdata(folder).get('object_provides'))
        self.assertIn(IRestorable.__identifier__,
                      self.get_catalog_indexdata(folder).get('object_provides'))

        self.assertTrue(self.get_catalog_indexdata(subfolder).get('trashed'))
        self.assertIn(ITrashed.__identifier__,
                      self.get_catalog_indexdata(subfolder).get('object_provides'))
        self.assertNotIn(IRestorable.__identifier__,
                         self.get_catalog_indexdata(subfolder).get('object_provides'))

        Trasher(folder).restore()
        self.assertFalse(self.get_catalog_indexdata(folder).get('trashed'))
        self.assertNotIn(ITrashed.__identifier__,
                         self.get_catalog_indexdata(folder).get('object_provides'))
        self.assertNotIn(IRestorable.__identifier__,
                         self.get_catalog_indexdata(folder).get('object_provides'))

        self.assertFalse(self.get_catalog_indexdata(subfolder).get('trashed'))
        self.assertNotIn(ITrashed.__identifier__,
                         self.get_catalog_indexdata(subfolder).get('object_provides'))
        self.assertNotIn(IRestorable.__identifier__,
                         self.get_catalog_indexdata(subfolder).get('object_provides'))

    def test_cannot_restore_content_when_not_trashed(self):
        self.grant('Site Administrator')

        folder = create(Builder('folder'))

        with self.assertRaises(NotRestorable):
            Trasher(folder).restore()

    def test_cannot_restore_content_when_parent_is_trashed(self):
        self.grant('Site Administrator')

        folder = create(Builder('folder'))
        subfolder = create(Builder('folder').within(folder))
        Trasher(folder).trash()

        with self.assertRaises(NotRestorable):
            Trasher(subfolder).restore()

    def test_cannot_restore_content_when_parent_was_trashed_earlier(self):
        self.grant('Site Administrator')

        folder = create(Builder('folder'))
        subfolder = create(Builder('folder').within(folder))
        Trasher(subfolder).trash()
        Trasher(folder).trash()

        with self.assertRaises(NotRestorable):
            Trasher(subfolder).restore()

    def test_restore_requires_permission(self):
        self.grant('Contributor')

        parent = create(Builder('folder'))
        parent.manage_permission('Restore trashed content', roles=[], acquire=False)

        folder = create(Builder('folder').within(parent))
        Trasher(folder).trash()
        self.assert_provides(folder, IRestorable, ITrashed)

        with self.assertRaises(Unauthorized):
            Trasher(folder).restore()

        self.assert_provides(folder, IRestorable, ITrashed)

        parent.manage_permission('Restore trashed content', roles=['Contributor'], acquire=False)
        Trasher(folder).restore()
        self.assert_provides(folder, None)

    def test_is_restorable_when_trashed(self):
        self.grant('Site Administrator')

        folder = create(Builder('folder'))
        self.assertFalse(Trasher(folder).is_restorable())

        Trasher(folder).trash()
        self.assertTrue(Trasher(folder).is_restorable())

    def test_not_restorable_when_parent_is_trashed_too(self):
        self.grant('Site Administrator')

        folder = create(Builder('folder'))
        subfolder = create(Builder('folder').within(folder))
        self.assertFalse(Trasher(folder).is_restorable())
        self.assertFalse(Trasher(subfolder).is_restorable())

        Trasher(subfolder).trash()
        self.assertFalse(Trasher(folder).is_restorable())
        self.assertTrue(Trasher(subfolder).is_restorable())

        Trasher(folder).trash()
        self.assertTrue(Trasher(folder).is_restorable())
        self.assertFalse(Trasher(subfolder).is_restorable())

    def test_trashing_and_restoring_updates_modified_date(self):
        self.grant('Site Administrator')

        created = datetime(2011, 1, 1, 1, 1, 1)
        trashed = datetime(2022, 2, 2, 2, 2, 2)
        restored = datetime(2033, 3, 3, 3, 3, 3)

        with freeze(created):
            folder = create(Builder('folder'))
            subfolder = create(Builder('folder').within(folder))

        self.assert_modified_date(created, folder)
        self.assert_modified_date(created, subfolder)

        with freeze(trashed):
            Trasher(folder).trash()

        self.assert_modified_date(trashed, folder)
        self.assert_modified_date(trashed, subfolder)

        with freeze(restored):
            Trasher(folder).restore()

        self.assert_modified_date(restored, folder)
        self.assert_modified_date(restored, subfolder)

    def test_trashing_and_restoring_parent_of_a_trashed_content_does_not_influence_child(self):
        """
        Given a subpage is trashed.
        When the parent page is also trashed, and later restored, the subpage should still
        be trashed.
        The subpage should also stay restorable.
        """
        self.grant('Site Administrator')

        folder = create(Builder('folder'))
        foo = create(Builder('folder').within(folder))
        bar = create(Builder('folder').within(folder))

        self.assert_provides(folder, None)
        self.assert_provides(foo, None)
        self.assert_provides(bar, None)

        Trasher(foo).trash()
        self.assert_provides(folder, None)
        self.assert_provides(foo, ITrashed, IRestorable)
        self.assert_provides(bar, None)

        Trasher(folder).trash()
        self.assert_provides(folder, ITrashed, IRestorable)
        self.assert_provides(foo, ITrashed, IRestorable)
        self.assert_provides(bar, ITrashed)

        Trasher(folder).restore()
        self.assert_provides(folder, None)
        self.assert_provides(foo, ITrashed, IRestorable)
        self.assert_provides(bar, None)

    def test_is_restorable_respectsIIsRestoreAllowedAdapter(self):
        response = {'allowed': True, 'called': 0}
        self.grant('Site Administrator')

        folder = create(Builder('folder'))
        Trasher(folder).trash()

        @implementer(IIsRestoreAllowedAdapter)
        @adapter(Interface, Interface)
        def is_restore_allowed(context, request):
            response['called'] += 1
            return response['allowed']

        getSiteManager().registerAdapter(is_restore_allowed)

        self.assertTrue(Trasher(folder).is_restorable())
        response['allowed'] = False
        self.assertFalse(Trasher(folder).is_restorable())
        self.assertEqual(2, response['called'])

    def test_events_are_fired(self):
        self.grant('Site Administrator')
        folder = create(Builder('folder'))
        fired_events = []
        registerHandler = self.portal.getSiteManager().registerHandler

        @registerHandler
        @adapter(Interface, IBeforeObjectTrashedEvent)
        def before_trashing(object, event):
            self.assertEqual(folder, object)
            self.assertEqual(folder, event.object)
            self.assertFalse(ITrashed.providedBy(object))
            fired_events.append(type(event).__name__)

        @registerHandler
        @adapter(Interface, IObjectTrashedEvent)
        def after_trashing(object, event):
            self.assertEqual(folder, object)
            self.assertEqual(folder, event.object)
            self.assertTrue(ITrashed.providedBy(object))
            fired_events.append(type(event).__name__)

        @registerHandler
        @adapter(Interface, IBeforeObjectRestoredEvent)
        def before_restoring(object, event):
            self.assertEqual(folder, object)
            self.assertEqual(folder, event.object)
            self.assertTrue(ITrashed.providedBy(object))
            fired_events.append(type(event).__name__)

        @registerHandler
        @adapter(Interface, IObjectRestoredEvent)
        def after_restoring(object, event):
            self.assertEqual(folder, object)
            self.assertEqual(folder, event.object)
            self.assertFalse(ITrashed.providedBy(object))
            fired_events.append(type(event).__name__)

        Trasher(folder).trash()
        self.assertEquals(['BeforeObjectTrashedEvent', 'ObjectTrashedEvent'], fired_events)
        fired_events[:] = []
        Trasher(folder).restore()
        self.assertEquals(['BeforeObjectRestoredEvent', 'ObjectRestoredEvent'], fired_events)


@duplicate_with_dexterity
class TestDefaultIsRestoreAllowedAdapter(FunctionalTestCase):

    def test_requires_add_permission_on_parent(self):
        self.grant('Contributor')
        parent = create(Builder('folder'))
        child = create(Builder('folder').within(parent))

        parent.manage_permission('Add portal content', roles=[], acquire=False)
        self.assertFalse(
            getMultiAdapter((child, self.request), IIsRestoreAllowedAdapter))

        parent.manage_permission('Add portal content', roles=['Contributor'], acquire=False)
        self.assertTrue(
            getMultiAdapter((child, self.request), IIsRestoreAllowedAdapter))



from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.testing import IS_PLONE_5
from ftw.testing.mailing import Mailing
from ftw.trash.interfaces import ITrashed
from ftw.trash.tests import FunctionalTestCase
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
import transaction


class TestPloneFormGen(FunctionalTestCase):
    def setUp(self):
        super(TestPloneFormGen, self).setUp()
        self.grant('Manager')

        # Set Missing Property, so mailing wont fail
        if IS_PLONE_5:
            from Products.CMFPlone.interfaces import IMailSchema
            registry = getUtility(IRegistry)
            mail_settings = registry.forInterface(IMailSchema, prefix='plone')
            mail_settings.email_from_address = 'foo@bar.ch'
        else:
            self.portal._updateProperty('email_from_address', 'foo@bar.ch')

    @browsing
    def test_ploneformgen_removes_trashed_fields(self, browser):
        form_folder = create(Builder('FormFolder'))

        reply_to = form_folder.get('replyto')
        self.assertIn(reply_to, form_folder._getFieldObjects())

        browser.login().visit(form_folder)
        self.assertTrue(browser.find('Your E-Mail Address'))

        form_folder.manage_delObjects([reply_to.id])
        transaction.commit()

        self.assertTrue(ITrashed.providedBy(reply_to))
        self.assertNotIn(reply_to, form_folder._getFieldObjects())

        browser.reload()
        self.assertFalse(browser.find('Your E-Mail Address'))

    @browsing
    def test_ploneformgen_field_not_required(self, browser):
        form_folder = create(Builder('FormFolder'))
        form_folder.manage_delObjects(['replyto'])
        transaction.commit()

        browser.login().visit(form_folder)
        browser.find_form_by_field('Subject').fill({
            'Subject': 'Foo',
             'Comments': 'Bar'
        }).submit()

        self.assertNotIn('This field is required.', browser.css('.fieldErrorBox').text)

    @browsing
    def test_ploneformgen_adapters_dont_run(self, browser):
        Mailing(self.layer['portal']).set_up()

        form_folder = create(Builder('FormFolder'))
        form_folder.manage_delObjects(['mailer'])
        transaction.commit()

        browser.login().visit(form_folder)
        browser.find_form_by_field('Subject').fill({
            'Your E-Mail Address': 'foo@bar.ch',
            'Subject': 'Foo',
            'Comments': 'Bar'
        }).submit()

        self.assertFalse(Mailing(self.portal).has_messages())



from zope.component.interfaces import IObjectEvent
from zope.interface import Interface


class IRestorable(Interface):
    """Marker interface for objects which can be restored.
    """


class ITrashed(Interface):
    """Marker interface for trashed objects which should no longer appear.
    """


class IIsRestoreAllowedAdapter(Interface):
    """The IIsRestoreAllowedAdapter multi adapter decides whether an object can be
    restored or not.
    See the readme for usage details.
    """


class IBeforeObjectTrashedEvent(IObjectEvent):
    """An object will be trashed.
    """


class IObjectTrashedEvent(IObjectEvent):
    """An object has been trashed.
    """


class IBeforeObjectRestoredEvent(IObjectEvent):
    """An object will be restored.
    """


class IObjectRestoredEvent(IObjectEvent):
    """An object has been restored.
    """



from ftw.builder import builder_registry
from ftw.builder.dexterity import DexterityBuilder
from ftw.builder.archetypes import ArchetypesBuilder


class DXFolderBuilder(DexterityBuilder):
    portal_type = 'dxfolder'


class FormFolderBuilder(ArchetypesBuilder):
    portal_type = 'FormFolder'


builder_registry.register('FormFolder', FormFolderBuilder)



from Acquisition import aq_inner
from Acquisition import aq_parent
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import plone
from ftw.trash.testing import TRASH_NOT_INSTALLED_FUNCTIONAL
from ftw.trash.tests import duplicate_with_dexterity
from ftw.trash.tests import FunctionalTestCase
from Products.CMFCore.utils import getToolByName
from zExceptions import Unauthorized


@duplicate_with_dexterity
class TestTrashNotInstalled(FunctionalTestCase):
    layer = TRASH_NOT_INSTALLED_FUNCTIONAL

    @browsing
    def test_content_is_deleted_when_trash_not_installed(self, browser):
        catalog = getToolByName(self.layer['portal'], 'portal_catalog')
        self.grant('Contributor')

        folder = create(Builder('folder').within(create(Builder('folder'))))
        self.assertIn(folder.getId(), aq_parent(aq_inner(folder)).objectIds())
        self.assertEqual(2, len(catalog.unrestrictedSearchResults()))

        browser.login().visit(folder)
        browser.click_on('Delete')
        self.assertEquals('Do you really want to delete this folder and all its contents?',
                          plone.first_heading())
        browser.click_on('Delete')
        self.assertNotIn(folder.getId(), aq_parent(aq_inner(folder)).objectIds())
        self.assertEqual(1, len(catalog.unrestrictedSearchResults()))

    @browsing
    def test_site_root_content_is_deleted_when_trash_not_installed(self, browser):
        catalog = getToolByName(self.layer['portal'], 'portal_catalog')
        self.grant('Contributor')

        folder = create(Builder('folder'))
        self.assertIn(folder.getId(), aq_parent(aq_inner(folder)).objectIds())
        self.assertEqual(1, len(catalog.unrestrictedSearchResults()))

        browser.login().visit(folder)
        browser.click_on('Delete')
        self.assertEquals('Do you really want to delete this folder and all its contents?',
                          plone.first_heading())
        browser.click_on('Delete')
        self.assertNotIn(folder.getId(), aq_parent(aq_inner(folder)).objectIds())
        self.assertEqual(0, len(catalog.unrestrictedSearchResults()))

    def test_manage_delObjects_requires_both_delete_permissions(self):
        """collective.deletepermission adapts the behavior so that a user needs both,
        "Delete objects" on the parent and "Delete portal content" on the child.
        If this changes, we must adapt our code accordingly, therefore this test.
        So: dont only change the test but our code too.

        This test must work accordingly to
        ftw.trash.tests.test_deletion.TestDeletion.[test name]
        """

        self.grant('Manager')
        parent = create(Builder('folder'))
        parent.manage_permission('Delete objects', roles=['Contributor'], acquire=False)
        child = create(Builder('folder').within(parent))
        child.manage_permission('Delete portal content', roles=['Contributor'], acquire=False)
        self.assertIn(child.getId(), parent.objectIds())
        self.assert_provides(parent, None)

        with self.assertRaises(Unauthorized):
            # Our test user with role Manager is not allowed to delete the child because
            # we have limited the permissions to "Contributor".
            parent.manage_delObjects([child.getId()])
            self.assertIn(child.getId(), parent.objectIds())
            self.assert_provides(parent, None)

        user = create(Builder('user').with_roles('Contributor', on=parent))
        with self.user(user):
            parent.manage_delObjects([child.getId()])
            self.assertNotIn(child.getId(), parent.objectIds())
            self.assert_provides(parent, None)



from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from contextlib import contextmanager
from DateTime import DateTime
from ftw.builder import builder_registry
from ftw.testing import IS_PLONE_5
from ftw.trash.interfaces import IRestorable
from ftw.trash.interfaces import ITrashed
from ftw.trash.testing import TRASH_FUNCTIONAL
from ftw.trash.tests.builders import DXFolderBuilder
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from Products.CMFCore.utils import getToolByName
from unittest2 import TestCase
import sys
import transaction


class FunctionalTestCase(TestCase):
    layer = TRASH_FUNCTIONAL

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def grant(self, *roles):
        setRoles(self.portal, TEST_USER_ID, list(roles))
        transaction.commit()

    @contextmanager
    def user(self, username):
        if hasattr(username, 'getUserName'):
            username = username.getUserName()

        sm = getSecurityManager()
        login(self.layer['portal'], username)
        try:
            yield
        finally:
            setSecurityManager(sm)

    def get_catalog_indexdata(self, obj):
        """Return the catalog index data for an object as dict.
        """
        self.maybe_process_indexing_queue()
        catalog = getToolByName(self.portal, 'portal_catalog')
        rid = catalog.getrid('/'.join(obj.getPhysicalPath()))
        return catalog.getIndexDataForRID(rid)

    def get_catalog_metadata(self, obj):
        """Return the catalog metadata for an object as dict.
        """
        self.maybe_process_indexing_queue()
        catalog = getToolByName(self.portal, 'portal_catalog')
        rid = catalog.getrid('/'.join(obj.getPhysicalPath()))
        return catalog.getMetadataForRID(rid)

    def assert_modified_date(self, expected, obj):
        expected = DateTime(expected)
        self.assertEqual(expected, obj.modified(), '{!r}.modified() is wrong'.format(obj))
        self.assertEqual(expected, self.get_catalog_metadata(obj)['modified'],
                         '{!r}\'s "modified" metadata is wrong'.format(obj))

        catalog = getToolByName(self.portal, 'portal_catalog')
        modified_index = catalog._catalog.indexes.get('modified')

        self.assertEqual(
            modified_index._convert(expected), self.get_catalog_indexdata(obj)['modified'],
            '{!r}\'s "modified" index data is wrong'.format(obj))

    def assert_provides(self, obj, *expected):
        expected = set(filter(None, expected))
        got = {iface for iface in (IRestorable, ITrashed) if iface.providedBy(obj)}
        self.assertEquals(expected, got, 'Unexpected interfaces provided by {!r}'.format(obj))

    @property
    def is_dexterity(self):
        return IS_PLONE_5

    def maybe_process_indexing_queue(self):
        if not IS_PLONE_5:
            return

        from Products.CMFCore.indexing import processQueue
        processQueue()


def duplicate_with_dexterity(klass):
    """Decorator for duplicating a test suite to be ran against dexterity contents.

    The tests are ran against archetypes by default, meaning that we use the builder
    "folder" as AT builder for the FTI "Folder".
    When using the @duplicate_with_dexterity decorator, an additional class is registered
    (postfixed "Dexterity"), where the "folder" builder is changed to a DX builder
    creating the FTI "dxfolder", which is registered in a Generic Setup test-profile.

    So if you use this decorator, be aware, that only "Builder('folder')" is replaced
    with dexterity, all other builders and other code does not change.
    """

    if IS_PLONE_5:
        # The default types (Folder etc.) in Plone 5 are already Dexterity.
        # So we do not test Archetypes under Plone 5 anymore, thus we do not
        # need to duplicate the tests.
        return klass

    class DexterityTestSuite(klass):
        def setUp(self):
            self._builder_isolation = builder_registry.temporary_builder_config()
            self._builder_isolation.__enter__()
            builder_registry.register('folder', DXFolderBuilder, force=True)
            super(DexterityTestSuite, self).setUp()

        def tearDown(self):
            super(DexterityTestSuite, self).tearDown()
            self._builder_isolation.__exit__(None, None, None)
            del self._builder_isolation

        @property
        def is_dexterity(self):
            return True

    DexterityTestSuite.__name__ = klass.__name__ + 'Dexterity'
    DexterityTestSuite.__module__ = klass.__module__
    sys._getframe(1).f_locals[DexterityTestSuite.__name__] = DexterityTestSuite
    return klass



from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import isLinked
from zope.component.hooks import getSite
import inspect
import os


def is_trash_profile_installed():
    portal_setup = getToolByName(getSite(), 'portal_setup')
    return portal_setup.getLastVersionForProfile('ftw.trash:default') != 'unknown'


def within_link_integrity_check():
    """Returns True when we are within a link integrity check.
    """
    frame = inspect.currentframe()
    while True:
        frame = frame.f_back
        if frame is None:
            return False

        if frame.f_code == isLinked.func_code:
            return True


def called_from_ZMI(request):
    """Returns True when the object deletion was called from ZMI.
    """
    if request is None:
        return None
    return request.PUBLISHED.__name__ == 'manage_delObjects'


def filter_children_in_paths(paths):
    """Accepts a list of paths and returns a filtered list, where children are removed
    when their parents (or grandparents, etc.) are also included in the list.
    As a side effect, trailing slashes are removed.
    """
    paths = map(lambda path: os.path.join(path, ''), sorted(paths, reverse=True))
    for parent in paths[:]:
        for child in paths[:]:
            if parent != child and child.startswith(parent):
                paths.remove(child)
    return map(lambda path: path.rstrip('/'), sorted(paths))



from Products.CMFCore.utils import getToolByName


def uninstall(self):
    setup_tool = getToolByName(self, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile(
        'profile-ftw.trash:uninstall',
        ignore_dependencies=True)



from ftw.trash.testing import TRASH_NOT_INSTALLED_FUNCTIONAL
from ftw.trash.tests import FunctionalTestCase
from ftw.trash.utils import filter_children_in_paths
from ftw.trash.utils import is_trash_profile_installed
from unittest2 import TestCase


class TestUtilsWhenInstalled(FunctionalTestCase):

    def test_is_trash_profile_installed(self):
        self.assertTrue(is_trash_profile_installed())


class TestUtilsWhenNotInstalled(FunctionalTestCase):
    layer = TRASH_NOT_INSTALLED_FUNCTIONAL

    def test_is_trash_profile_installed(self):
        self.assertFalse(is_trash_profile_installed())


class TestFilterChildrenInPaths(TestCase):

    def test(self):
        paths = ['/a/b/c/d', '/a', '/b', '/a/b/c/d/e/f', '/c/a/b/d', '/abab']
        self.assertEquals(['/a', '/abab', '/b', '/c/a/b/d'],
                          filter_children_in_paths(paths))



from datetime import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import plone
from ftw.testbrowser.pages import statusmessages
from ftw.testing import freeze
from ftw.testing import IS_PLONE_5
from ftw.trash.interfaces import IRestorable
from ftw.trash.interfaces import ITrashed
from ftw.trash.tests import duplicate_with_dexterity
from ftw.trash.tests import FunctionalTestCase
from ftw.trash.trasher import Trasher
import transaction


@duplicate_with_dexterity
class TestTrashView(FunctionalTestCase):

    @browsing
    def test_view_requires_restore_permission(self, browser):
        self.grant('Contributor')
        self.portal.manage_permission('Restore trashed content', roles=[], acquire=False)
        transaction.commit()

        trash_link_label = 'Trash'

        browser.login().open()
        self.assertNotIn(trash_link_label, browser.css('#portal-personaltools li').text)
        with browser.expect_unauthorized():
            browser.open(view='trash')

        self.portal.manage_permission('Restore trashed content', roles=['Contributor'],
                                      acquire=False)
        transaction.commit()

        browser.open()
        self.assertIn(trash_link_label, browser.css('#portal-personaltools li').text)
        browser.css('#portal-personaltools li').find(trash_link_label).first.click()
        self.assertEquals(self.portal.portal_url() + '/trash', browser.url)

    @browsing
    def test_lists_only_restorable_content(self, browser):
        self.grant('Site Administrator')

        create(Builder('folder').titled(u'Keep that'))
        parent = create(Builder('folder').titled(u'Parent'))
        folder = create(Builder('folder').titled(u'Delete that').within(parent))
        create(Builder('folder').titled(u'Child of folder to be deleted').within(folder))

        with freeze(datetime(2016, 12, 15, 17, 9)):
            Trasher(folder).trash()

        transaction.commit()
        browser.login().open().click_on('Trash')
        self.assertEquals('trash', plone.view())

        self.assertEquals(
            [{'Last modified': 'Dec 15, 2016 05:09 PM',
              'Type': self.type_label,
              'Title': 'Delete that http://nohost/plone/parent/delete-that',
              '': ''}],
            browser.css('.trash-table').first.dicts())

    @browsing
    def test_restore_trashed_content(self, browser):
        self.grant('Site Administrator')

        folder = create(Builder('folder').titled(u'My Folder'))
        with freeze(datetime(2011, 1, 1)):
            Trasher(folder).trash()

        self.assert_provides(folder, IRestorable, ITrashed)

        transaction.commit()
        browser.login().open().click_on('Trash')

        self.assertEquals(
            [{'Last modified': 'Jan 01, 2011 12:00 AM',
              'Type': self.type_label,
              'Title': 'My Folder http://nohost/plone/my-folder',
              '': ''}],
            browser.css('.trash-table').first.dicts())

        browser.css('.trash-table').find('Restore').first.click()
        statusmessages.assert_message('The content "My Folder" has been restored.')
        self.assertEqual(folder.absolute_url(), browser.url)

        transaction.begin()
        self.assert_provides(folder, None)

    @browsing
    def test_restore_redirects_to_view_for_files(self, browser):
        self.grant('Site Administrator')
        doc = create(Builder('file').with_dummy_content())
        Trasher(doc).trash()
        transaction.commit()
        browser.login().open(view='trash').click_on('Restore')
        self.assertEquals('http://nohost/plone/{}/view'.format(doc.getId()), browser.url)

    @browsing
    def test_error_message_when_restore_not_allowed(self, browser):
        self.grant('Site Administrator')

        parent = create(Builder('folder').titled(u'Parent'))
        folder = create(Builder('folder').titled(u'My Folder').within(parent))
        Trasher(folder).trash()
        parent.manage_permission('Add portal content', roles=[], acquire=False)
        transaction.commit()

        browser.login().open().click_on('Trash').click_on('Restore')
        statusmessages.assert_message(
            'You are not allowed to restore "My Folder".'
            ' You may need to change the workflow state of the parent content.')

        transaction.begin()
        self.assert_provides(folder, IRestorable, ITrashed)

    @browsing
    def test_error_message_when_parent_is_trashed(self, browser):
        self.grant('Site Administrator')

        parent = create(Builder('folder').titled(u'Parent'))
        child = create(Builder('folder').titled(u'Child').within(parent))
        Trasher(parent).trash()
        Trasher(child).trash()
        self.assert_provides(parent, IRestorable, ITrashed)
        self.assert_provides(child, IRestorable, ITrashed)
        transaction.commit()

        browser.login().open().click_on('Trash')
        table = browser.css('table.trash-table')
        table.xpath('//*[text()="Child"]').first.parent('tr').find('Restore').click()

        statusmessages.assert_message(
            '"Child" cannot be restored because the parent container "Parent" is also'
            ' trashed. You need to restore the parent first.')

        transaction.begin()
        self.assert_provides(parent, IRestorable, ITrashed)
        self.assert_provides(child, IRestorable, ITrashed)

    @browsing
    def test_empy_trash(self, browser):
        self.grant('Site Administrator')

        Trasher(create(Builder('folder').titled(u'Foo'))).trash()
        self.assertIn('foo', self.portal.objectIds())
        transaction.commit()

        browser.login().open().click_on('Trash')
        self.assertEquals('Trash', plone.first_heading())

        browser.click_on('Clean trash')
        self.assertEquals(
            'Are you sure you want to permanently delete all objects in the trash?',
            plone.first_heading())
        self.assertIn('foo', self.portal.objectIds())

        browser.click_on('Cancel')
        self.assertEquals('Trash', plone.first_heading())
        self.assertIn('foo', self.portal.objectIds())

        browser.click_on('Clean trash').click_on('Delete')
        self.assertEquals('Trash', plone.first_heading())
        self.assertNotIn('foo', self.portal.objectIds())

    @browsing
    def test_empty_trash_button_not_visible_without_permission(self, browser):
        self.grant('Site Administrator')
        browser.login().open().click_on('Trash')

        self.assertTrue(browser.find('Clean trash'))

        self.portal.manage_permission('Clean trash', roles=[], acquire=False)
        transaction.commit()

        browser.reload()
        self.assertFalse(browser.find('Clean trash'))

    @browsing
    def test_empty_trash_protected_by_permission(self, browser):
        self.grant('Site Administrator')
        browser.login().open().click_on('Trash')

        browser.click_on('Clean trash')

        self.portal.manage_permission('Clean trash', roles=[], acquire=False)
        transaction.commit()

        with browser.expect_unauthorized():
            browser.click_on('Delete')

    @browsing
    def test_empty_trash_when_parent_and_child_are_both_trashed(self, browser):
        """Regression test: when trying to delete a child but the parent was deleted
        bevore, an exception is raised.
        Solution: do not delete children and parents in the same set; just delete parents,
        which is recursive anyway.
        """
        self.grant('Site Administrator')

        parent = create(Builder('folder').titled(u'Parent'))
        child = create(Builder('folder').titled(u'Child').within(parent))

        Trasher(child).trash()
        Trasher(parent).trash()
        transaction.commit()

        browser.login().open().click_on('Trash')
        self.assertEquals('Trash', plone.first_heading())

        self.assertIn('parent', self.portal.objectIds())
        browser.click_on('Clean trash').click_on('Delete')
        self.assertNotIn('parent', self.portal.objectIds())

    @browsing
    def test_empty_trash_event_if_user_has_insufficient_permissions_on_the_item(self, browser):
        self.grant('Site Administrator')

        folder = create(Builder('folder').within(create(Builder('folder'))))
        parent = folder.aq_parent
        folder_id = folder.getId()

        parent.manage_delObjects([folder_id])
        folder.manage_permission('Delete portal content', roles=['Manager'], acquire=False)
        transaction.commit()

        browser.login().open().click_on('Trash')
        browser.click_on('Clean trash').click_on('Delete')

        self.assertNotIn(folder_id, parent)

    @property
    def type_label(self):
        if self.is_dexterity and not IS_PLONE_5:
            return 'dxfolder'
        else:
            return 'Folder'



from ftw.builder import Builder
from ftw.builder import create
from ftw.trash.tests import duplicate_with_dexterity
from ftw.trash.tests import FunctionalTestCase
from ftw.trash.trasher import Trasher
from Products.CMFCore.utils import getToolByName


@duplicate_with_dexterity
class TestCatalog(FunctionalTestCase):

    def test_filters_trashed_content_by_default(self):
        self.grant('Contributor')

        create(Builder('folder').titled(u'Foo'))
        bar = create(Builder('folder').titled(u'Bar'))
        self.assert_catalog_result({'Bar', 'Foo'})

        Trasher(bar).trash()
        self.assert_catalog_result({'Foo'})

    def test_returns_trashed_content_when_explicitly_queried_for(self):
        self.grant('Contributor')

        create(Builder('folder').titled(u'Foo'))
        Trasher(create(Builder('folder').titled(u'Bar'))).trash()
        self.assert_catalog_result({'Foo'}, query={})
        self.assert_catalog_result({'Foo', 'Bar'}, query={'trashed': None})
        self.assert_catalog_result({'Bar'}, query={'trashed': True})
        self.assert_catalog_result({'Foo'}, query={'trashed': False})

    def test_unrestrictedSearchResults_returns_all_items_by_default(self):
        """unrestrictedSearchResults must return all items by default, no matter whether
        they're trashed or not, because ftw.upgrade uses unrestrictedSearchResults
        and we want to make sure that all objects are migrated.
        """
        self.grant('Contributor')

        create(Builder('folder').titled(u'Foo'))
        Trasher(create(Builder('folder').titled(u'Bar'))).trash()
        catalog = getToolByName(self.portal, 'portal_catalog')
        self.assertItemsEqual(('Foo', 'Bar'),
                              [brain.Title for brain in catalog.unrestrictedSearchResults()])

    def assert_catalog_result(self, expected_titles, query={}):
        catalog = getToolByName(self.portal, 'portal_catalog')
        got_titles1 = [brain.Title for brain in catalog.searchResults(**query)]
        got_titles2 = [brain.Title for brain in catalog(**query)]

        self.assertItemsEqual(
            got_titles1, got_titles2,
            'portal_catalog({!r}) != portal_catalog.searchResults({!r})'.format(query, query))

        self.assertItemsEqual(expected_titles, got_titles1)



import logging
from Products.CMFCore.utils import getToolByName


PROFILE_ID = 'profile-ftw.trash:default'
INDEXES = (('trashed', 'FieldIndex'),)


def installed(site):
    add_catalog_indexes(site)


def uninstalled(site):
    remove_catalog_indexes(site)


def add_catalog_indexes(context, logger=None):
    """Method to add our wanted indexes to the portal_catalog.

    @parameters:

    When called from the import_various method below, 'context' is
    the plone site and 'logger' is the portal_setup logger.  But
    this method can also be used as upgrade step, in which case
    'context' will be portal_setup and 'logger' will be None.
    """
    if logger is None:
        # Called as upgrade step: define our own logger.
        logger = logging.getLogger('ftw.trash')

    # Run the catalog.xml step as that may have defined new metadata
    # columns.  We could instead add <depends name="catalog"/> to
    # the registration of our import step in zcml, but doing it in
    # code makes this method usable as upgrade step as well.  Note that
    # this silently does nothing when there is no catalog.xml, so it
    # is quite safe.
    setup = getToolByName(context, 'portal_setup')
    setup.runImportStepFromProfile(PROFILE_ID, 'catalog')

    catalog = getToolByName(context, 'portal_catalog')
    indexes = catalog.indexes()
    # Specify the indexes you want, with ('index_name', 'index_type')
    wanted = INDEXES
    indexables = []
    for name, meta_type in wanted:
        if name not in indexes:
            catalog.addIndex(name, meta_type)
            indexables.append(name)
            logger.info("Added %s for field %s.", meta_type, name)
    if len(indexables) > 0:
        logger.info("Indexing new indexes %s.", ', '.join(indexables))
        catalog.manage_reindexIndex(ids=indexables)


def remove_catalog_indexes(context):
    catalog = getToolByName(context, 'portal_catalog')
    indexes = catalog.indexes()

    for name, meta_type in INDEXES:
        if name in indexes:
            catalog.delIndex(name)



from plone.autoform.interfaces import IFormFieldProvider
from zope.interface import alsoProvides
from zope.interface import Interface


class IDXFolder(Interface):
    pass

alsoProvides(IDXFolder, IFormFieldProvider)



from ftw.builder import Builder
from ftw.builder import create
from ftw.testbrowser import browsing
from ftw.trash.tests import FunctionalTestCase
from ftw.trash.trasher import Trasher


class TestZMI(FunctionalTestCase):

    @browsing
    def test_delete_from_ZMI_does_actually_delete_content(self, browser):
        self.grant('Manager')

        folder = create(Builder('folder').within(create(Builder('folder'))))
        subfolder = create(Builder('folder').within(folder))
        self.assertIn(subfolder.getId(), folder.objectIds())

        browser.login().open(folder, view='manage_main')
        self.assertEquals([('folder', 'folder')], browser.find('ids:list').options)
        browser.fill({'ids:list': 'folder'}).find('Delete').click()
        self.assertNotIn(subfolder.getId(), folder.objectIds())

    @browsing
    def test_manager_can_delete_trashed_content(self, browser):
        self.grant('Manager')

        folder = create(Builder('folder').within(create(Builder('folder'))))
        create(Builder('folder').within(folder).titled(u'Normal Folder'))
        Trasher(create(Builder('folder').within(folder).titled(u'Trashed Folder'))).trash()
        self.assertItemsEqual(['normal-folder', 'trashed-folder'], folder.objectIds())

        browser.login().open(folder, view='manage_main')
        self.assertEquals([('normal-folder', 'normal-folder'),
                           ('trashed-folder', 'trashed-folder')],
                          browser.find('ids:list').options)

        browser.fill({'ids:list': 'trashed-folder'}).find('Delete').click()
        self.assertItemsEqual(['normal-folder'], folder.objectIds())



from ftw.builder.testing import BUILDER_LAYER
from ftw.builder.testing import functional_session_factory
from ftw.builder.testing import set_builder_session_factory
from ftw.testing import IS_PLONE_5
from ftw.testing.layer import COMPONENT_REGISTRY_ISOLATION
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from zope.configuration import xmlconfig


class TrashLayer(PloneSandboxLayer):
    defaultBases = (COMPONENT_REGISTRY_ISOLATION, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '  <include package="ftw.trash.tests" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'ftw.trash')
        z2.installProduct(app, 'Products.PloneFormGen')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.trash:default')
        applyProfile(portal, 'ftw.trash.tests:dxtests')
        applyProfile(portal, 'Products.PloneFormGen:default')

        if IS_PLONE_5:
            applyProfile(portal, 'plone.app.contenttypes:default')


TRASH_FIXTURE = TrashLayer()
TRASH_FUNCTIONAL = FunctionalTesting(
    bases=(TRASH_FIXTURE,
           set_builder_session_factory(functional_session_factory)),
    name="ftw.trash:functional")


class TrashNotInstalledLayer(PloneSandboxLayer):
    """In the this layer, the ftw.trash package's ZCML is loaded but the Generic Setup
    profile is not installed.
    Loading of the ZCML is important as it would also happen in the production and it will
    apply patches, but the patches should only change behavior when also the Generic Setup
    profile is installed.
    """
    defaultBases = (COMPONENT_REGISTRY_ISOLATION, BUILDER_LAYER)

    def setUpZope(self, app, configurationContext):
        xmlconfig.string(
            '<configure xmlns="http://namespaces.zope.org/zope">'
            '  <include package="z3c.autoinclude" file="meta.zcml" />'
            '  <includePlugins package="plone" />'
            '  <includePluginsOverrides package="plone" />'
            '  <include package="ftw.trash.tests" />'
            '</configure>',
            context=configurationContext)

        z2.installProduct(app, 'ftw.trash')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'ftw.trash.tests:dxtests')
        applyProfile(portal, 'collective.deletepermission:default')

        if IS_PLONE_5:
            applyProfile(portal, 'plone.app.contenttypes:default')


TRASH_NOT_INSTALLED_FUNCTIONAL = FunctionalTesting(
    bases=(TrashNotInstalledLayer(),
           set_builder_session_factory(functional_session_factory)),
    name="ftw.trash:not-installed:functional")
